﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Selection;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  /// <summary>
  /// A class for selection operators which match a target to the scopes.
  /// </summary>
  [Item("MatchSelector", "A class for selection operators which match a target to the scopes.")]
  [StorableClass]
  public class MatchSelector : Selector, IMatchSelector {
    protected IValueParameter<BoolValue> CopySelectedParameter {
      get { return (IValueParameter<BoolValue>)Parameters["CopySelected"]; }
    }
    protected IValueParameter<BoolValue> MatchConditionParameter {
      get { return (IValueParameter<BoolValue>)Parameters["MatchCondition"]; }
    }
    public IValueLookupParameter<IntValue> NumberOfSelectedSubScopesParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["NumberOfSelectedSubScopes"]; }
    }
    public ILookupParameter<IClassifier> TargetMatchParameter {
      get { return (ILookupParameter<IClassifier>)Parameters["TargetMatchParameter"]; }
    }
    public ILookupParameter<ItemArray<IClassifier>> MatchParameter {
      get { return (ILookupParameter<ItemArray<IClassifier>>)Parameters["MatchParameter"]; }
    }
    public BoolValue CopySelected {
      get { return CopySelectedParameter.Value; }
      set { CopySelectedParameter.Value = value; }
    }

    public BoolValue MatchCondition {
      get { return MatchConditionParameter.Value; }
      set { MatchConditionParameter.Value = value; }
    }

    [StorableConstructor]
    protected MatchSelector(bool deserializing) : base(deserializing) { }
    protected MatchSelector(MatchSelector original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MatchSelector(this, cloner);
    }

    public MatchSelector()
      : base() {
      Parameters.Add(new ValueParameter<BoolValue>("CopySelected", "True if the selected sub-scopes should be copied, otherwise false.", new BoolValue(true)));
      Parameters.Add(new ValueParameter<BoolValue>("MatchCondition", "True if the condition of the match paramteres have to match, otherwise the action has to match.", new BoolValue(true)));
      Parameters.Add(new ValueLookupParameter<IntValue>("NumberOfSelectedSubScopes", "The number of sub-scopes which should be selected."));
      Parameters.Add(new ValueLookupParameter<IClassifier>("TargetMatchParameter", "Target that has to be matched by the match parameter."));
      Parameters.Add(new ScopeTreeLookupParameter<IClassifier>("MatchParameter", "The matching encoding contained in each sub-scope which is used for selection."));
    }

    protected override IScope[] Select(List<IScope> scopes) {
      IClassifier target = TargetMatchParameter.ActualValue;
      List<IScope> matched = new List<IScope>();
      for (int i = 0; i < MatchParameter.ActualValue.Length; i++) {
        if (MatchingMethod(MatchParameter.ActualValue[i], target)) {
          if (CopySelected.Value) {
            matched.Add((IScope)scopes[i].Clone());
          } else {
            matched.Add(scopes[i]);
          }
        }
      }
      NumberOfSelectedSubScopesParameter.ActualValue = new IntValue(matched.Count);
      return matched.ToArray();
    }

    private bool MatchingMethod(IClassifier iMatching, IClassifier target) {
      if (MatchCondition.Value) {
        return iMatching.MatchCondition(target);
      } else {
        return iMatching.MatchAction(target);
      }
    }
  }
}
